;===============================================================
; Yellow Parakoopa, Wave Motion
; Description: A yellow parakoopa which flies in wave motion
; instead of being just a normal koopa with wings and more hits.
;
; Uses first extra bit: YES
;===============================================================
; By Erik557
; NOTE: If you want to replace the original one,
; use the patch version instead.
;===============================================================
; Edited by SkywinDragoon
; Set extra bit to follow mario

YSpeeds:
       db $20,$E0
YAccel:
       db $01,$FF
XSpeeds:
       db $10,$F0

;=================================
; INIT and MAIN Wrappers
;=================================

print "INIT ",pc
       PHB : PHK : PLB      ; set data bank
       JSL $01ACF9          ;\ Get starting frame
       STA $1570,x          ;/
       JSR SubHorizPos      ;\
       TYA                  ; | Face player
       STA $157C,x          ;/
       LDA XSpeeds,y        ;\ Set initial X speed
       STA $B6,x            ;/
       PLB                  ; restore
RTL

print "MAIN ",pc
       PHB : PHK : PLB      ; set data bank
       JSR Override
       LDA $14C8,x
       CMP #$07
       BEQ YoshiSwallow
       JSR YellowParakoopa  ; Main routine
       PLB                  ; restore
RTL

;========================
; Main routine
;========================
Override:
	LDY #$01	; number of states to override -1
	LDA $14C8,x
-
	CMP .states,y
	BEQ .override
	DEY
	BPL -
	LDA $7FAB34,x
	AND #$7F
	STA $7FAB34,x
	RTS
.states			; statuses to override (including 08)
	db $07,$08
.override
	LDA $7FAB34,x
	ORA #$80
	STA $7FAB34,x
	RTS
YoshiSwallow:
	LDA #$07
           STA $9E,x
	JSL $07F7D2
	LDA #$07
           STA $14C8,x
	PLB
	RTL
DrawGFX:                    ;\ Draw the sprite
       PHB                  ; |\
       LDA #$01             ; | | set data bank to 01
       PHA                  ; | |
       PLB                  ; |/
       PHK                  ; |\
       PEA.w .drawn-1       ; | | jsl to 018bc3
       PEA.w $0180CA-1      ; | |
       JML $018BC3          ; |/
.drawn                      ;/
       PLB                  ; restore data bank
       RTS                  ; return

YellowParakoopa:
       LDY #$00
       JSR SubOffScreen
       LDA $9D
       BNE .LockedSprites

.NotTurning
       INC $1570,x          ;\
       LDA $1570,x          ; |
       LSR #3               ; | set animation frame
       AND #$01             ; |
       STA $1602,x          ;/

       JSL $018022          ; Update x position

       LDA $C2,x            ;\
       AND #$01             ; |
       TAY                  ; | Increase speed
       LDA $AA,x            ; |
       CLC                  ; |
       ADC YAccel,y         ; |
       STA $AA,x            ;/
       CMP YSpeeds,y        ;\
       BNE +                ; | If at max, change directions now
       INC $C2,x            ;/

+      JSL $01801A          ; Update y position

       LDA $7FAB10,x
       AND #$04             ; |
       BEQ .SkipFollow
       LDA $1570,x          ;\
       AND #$7F             ; | Return every certain frames
       BNE .DontFollow      ;/
       LDA $157C,x          ;\ Preserve current direction
       PHA                  ;/
       JSR SubHorizPos      ;\ Face Mario
       TYA                  ; |
       STA $157C,x          ;/
       PLA                  ;\
       CMP $157C,x          ; | If same direction as before, don't turn
       BEQ .DontFollow      ;/
       LDA $15AC,x          ;\ Don't set timer if it's already set
       BNE +                ;/
       LDA #$0F             ;\ Set turning image
       STA $15AC,x          ;/
+      DEC $B6,x            ; Decrease X speed (for a not too sudden change)

.DontFollow
       LDA $15AC,x          ;\
       CMP #$04             ; | Branch if timer not at specific time
       BNE .LockedSprites   ;/
       LDY $157C,x          ;\
       LDA XSpeeds,y        ; | Invert X speed
       STA $B6,x            ;/
.SkipFollow
.LockedSprites
       JSR DrawGFX          ; GFX routine
       JSL $018032          ; interact with sprites

       JSL $01A7DC          ;\
       BCS .Int             ; | interact with mario
-      JMP .ReturnInteract  ;/ if not in contact, return

.Int
       LDA $9D
       BNE -

       LDA $1490
       BEQ +
       JMP .Star

+      JSR CapeCollision
       BNE .Cape

       LDA $0E
       CMP #$E6
       BMI +
       JMP .Hurt

+      JSL $01AB99

       LDA $140D
       ORA $187A
       BEQ +
       JMP .SpinKill

+      JSL $01AA33
       LDA #$07
       STA $9E,x
       LDA $15F6,x
       AND #$0E
       STA $0D
       JSL $07F7D2
       LDA $15F6,x
       AND #$F1
       ORA $0D
       STA $15F6,x
       STZ $AA,x

       PHY
       LDA $1697
       CLC
       ADC $1626,X
       INC $1697
       TAY
       INY
       CPY #$08
       BCS +
       LDA KillSFX,y
       STA $1DF9
+      TYA
       CMP #$08
       BCC +
       LDA #$08
+      JSL $02ACE5
       PLY
       RTS

.Cape

       LDA $0E
       CMP #$35
       BEQ +
       JSL $01AB6F

+      LDA #$00
       JSL $02ACE5
       LDA #$09
       STA $14C8,x
       ASL $15F6,x
       SEC
       ROR $15F6,x
       LDA #$07
       STA $9E,x
       JSL $07F7D2

       LDA #$FF
       STA $1540,x
       STZ $1558,x
       LDA #$C0
       LDY $0E
       BEQ +
       LDA #$B0
       CPY #$02
       BNE +
       LDA #$C0
+      STA $AA,x

       JSR SubHorizPos
       LDA StunSpeeds,y
       STA $B6,x
       TYA
       EOR #$01
       STA $157C,x
       BRA .ReturnInteract

.Hurt
       JSL $00F5B7
       LDX $15E9
       JSR SubHorizPos
       TYA
       STA $157C,x
       BRA .ReturnInteract

.SpinKill
       LDA #$04
       STA $14C8,x
       LDA #$1F
       STA $1540,x
       LDA #$08
       STA $1DF9
       JSL $07FC3B
       LDA #$F8
       STA $7D

.ReturnInteract
       RTS                  ; return

.Star
       JSL $01AB6F
       INC $18D2
       LDA $18D2
       CMP #$08
       BCC +
       LDA #$08
       STA $18D2
+      JSL $02ACE5
       LDY $18D2
       CPY #$08
       BCS +
       LDA KillSFX,y
       STA $1DF9
+      LDA #$02
       STA $14C8,x
       LDA #$D0
       STA $AA,x
       JSR SubHorizPos
       LDA KillSpeeds,y
       STA $B6,x
       LDA #$07
       STA $9E,x
       RTS

KillSFX:
       db $00,$13,$14,$15,$16,$17,$18,$19
KillSpeeds:
       db $F0,$10
StunSpeeds:
       db $F8,$08

;==================
  SubHorizPos:
;==================

       LDY #$00
       LDA $94
       SEC
       SBC $E4,x
       STA $0F
       LDA $95
       SBC $14E0,x
       BPL $01
       INY
       RTS 

;==================
  CapeCollision:
;==================

       LDA $13E8
       BEQ .NoContact
       LDA $15D0,x
       ORA $1FE2,x
       BNE .NoContact
       LDA $1632,x
       PHY
       LDY $74
       BEQ +
       EOR #$01
+      PLY
       EOR $13F9 
       BNE .NoContact
       JSL $03B69F
       LDA $13E9
       SEC
       SBC #$02
       STA $00
       LDA $13EA
       SBC #$00
       STA $08
       LDA #$14
       STA $02
       LDA $13EB
       STA $01
       LDA $13EC
       STA $09
       LDA #$10
       STA $03
       JSL $03B72B
       BCC .NoContact
       LDA #$01
      RTS
.NoContact
       LDA #$00
       RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  SubOffScreen:           ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       
       LDA $15A0,x
       ORA $186C,x
       BEQ .Return2
       LDA $167A,x
       AND #$04
       BNE .Return2
       LDA $5B
       LSR A
       BCS .VerticalLevel
       LDA $D8,x
       ADC #$50
       LDA $14D4,x
       ADC #$00
       CMP #$02
       BCS .EraseSprite
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC .AddTable,y
       SEC
       SBC $1A
       CMP .CmpTable,y
.Common
       SEP #$20
       BCC .Return2
.EraseSprite
       LDA $14C8,x
       CMP #$08
       BCC .KillSprite
       LDY $161A,x
       CPY #$FF
       BEQ .KillSprite
       LDA $1938,y
       AND #$FE
       STA $1938,y
.KillSprite
       STZ $14C8,x
.Return2
       RTS

.VerticalLevel
       LDA $13
       LSR A
       BCS .CheckY
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC.w #$0040
       CMP.w #$0280
       BRA .Common
.CheckY
       LDA $14D4,x
       XBA
       LDA $D8,x
       REP #$20
       SBC $1C
       CLC
       ADC.w #$0070
       CMP.w #$01D0
       BRA .Common

.AddTable
       dw $0040,$0040,$0010,$0070
       dw $0090,$0050,$0080,$FFC0
.CmpTable
       dw $0170,$01E0,$01B0,$01D0
       dw $0230,$01B0,$0220,$0160

